@@ -4,6 +4,10 @@
     SPDX-FileCopyrightText: 2021 Waqar Ahmed <waqar.17a@gmail.com>
     SPDX-License-Identifier: LGPL-2.0-or-later
 */
+
+#ifndef CAMELCURSORTEST_H
+#define CAMELCURSORTEST_H
+
 #include <QObject>
 
 namespace KTextEditor
@@ -34,3 +38,5 @@ private:
     KTextEditor::ViewPrivate *view;
     KTextEditor::DocumentPrivate *doc;
 };
+
+#endif
